# ----------------------------------------------------------------------------
#       Copyright (C) 2013-2014 Huynh Vi Lam  <domovilam@gmail.com>
#
#       This file is part of pimucha.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import datetime,logging
from piHAparsers.x10libs import (checkhu)
from piHAlibs import (
    dumprows,majdb3,ralldb3,reqdb3,
    newalias,newhu
)
from .aliaslibs import (huorhexid)

logger = logging.getLogger()


def addunit(data):
    """
    item = [typeunit,unitid,protocol,hu,alias]
    typeunit = ['detector','device','remote']
    unitid = addrid (hex form) or hu (houseunit)
    alias = name / label of the unit
    """
    if isinstance(data,list):
        item = data
    else:
        item = data.split()
    if len(item) < 4: return 'Minimum of four parameters needed'
    tu = item[0].lower()
    if tu not in ['detector','device','remote']: return 'Type unit incorrect for this function'
    unitid = huorhexid(item[1])
    protocol = item[2].lower()
    if protocol not in ['x10','x10s','arctech']: return 'Protocol incorrect for this function'
    if not checkhu(item[3]): return 'Syntax error for houseunit code %s' % repr(item[3])
    hu = item[3].upper()
    if len(item) == 4:
        alias = tu + ' ' + hu
    else:
        alias = ' '.join(item[4:])
    alias = alias.replace(' ','_')
    d = datetime.date.strftime(datetime.datetime.today(),"%Y-%m-%d %H:%M:%S").split()
    req = "INSERT INTO unitIds(cdate,ctime,typeunit,unitId,iddecoder,unit,label,enabled) "
    req += " VALUES (?,?,?,?,?,?,?,?);"
    req1 = "INSERT INTO allunitIds(cdate,ctime,typeunit,unitId,iddecoder,unit,label,enabled) "
    req1 += " VALUES (?,?,?,?,?,?,?,?);"
    V = [d[0], d[1], tu, unitid, protocol, hu, alias,'True']
    try:
        majdb3(req,V)
        newalias([tu, hu, alias])
        majdb3(req1,V)
    except Exception as e:
        logger.error("Update failed for newids with error : %s",repr(e))
        return "Update failed for newids with error : %s",repr(e)
    return 'Unit %s added successfully' % item[1]


def chgunit(data):
    """
    Affect to an new unitid an existing hu (and alias)
    Remove the new hu (and alias)
    item = [typeunit,unitid,oldhu,(alias)]
    typeunit = ['detector','device','sensor','remote']
    new unitid (addrid hex form or houseunit) must exist : unitid
    if alias not provided, oldhu must exist (alias exists also)
    if alias provided, add alias and hu
    """
    if isinstance(data,list):
        item = data
    else:
        item = data.split()
    if len(item) < 3: return 'Minimun three parameters needed'
    tu = item[0].lower()
    if tu not in ['detector','device','sensor','remote']: return 'Type unit incorrect for this function'
    oldhu = item[2].upper()
    if not checkhu(oldhu): return 'Syntax error for houseunit code %s' % repr(oldhu)
    #if alias provided
    if len(item) > 3:
        alias = ' '.join(item[3:])
        alias = alias.replace(' ','_')
    else:
        #Check alias
        req = "SELECT rowid,label FROM alias WHERE typeunit = ? AND unit = ?  ;"
        V = [tu,oldhu]
        r1 = reqdb3(req,V)
        if r1 is None:
            return 'Unit %s must exist in alias' % oldhu
        else:
            alias = r1[1]
    #Add alias and hu provided
    newalias([tu, oldhu, alias])
    #Check new unitId
    unitid = item[1]
    req = "SELECT rowid FROM allunitIds WHERE typeunit = ? AND unitId = ?  ;"
    V = [tu,unitid]
    r = reqdb3(req,V)
    if r is None: return 'UnitId %s must exist in allunitids' % unitid
    rid = r[0]
    #delete previous record of unitid if exists
    req = "DELETE FROM unitIds WHERE unitId = ? ;"
    majdb3(req,(unitid,))
    #enable unitid
    enable([item[0],str(rid)])
    #update unit of unitid
    req = "UPDATE unitIds SET unit = ? WHERE rowId = ?"
    majdb3(req,(oldhu,rid))
    #Remove alias if exists
    req = "DELETE FROM alias WHERE unit = ?;"
    V = [unitid,]
    majdb3(req,V)
    return 'Changes successful for %s' % data


def lsunit(param,rowids=None):
    param = param.lower()
    if param not in ['allparam','allunit','alias','detector','device','parameter','sensor','remote']: param = 'alias'
    req = "SELECT "
    if param == 'alias':
        req +=  " d.rowId,unit,label,typeunit,enabled FROM alias as d "
        if rowids: req += " WHERE d.rowId IN (" + rowids + ")"
    elif param == 'allunit':
        req +=  " d.rowId,unitId,iddecoder,d.unit,a.label,cdate,ctime,modelId,d.enabled "
        req +=  " FROM allunitIds as d LEFT JOIN alias as a ON d.unit = a.unit "
    elif param in ('detector','device','sensor','remote'):
        req +=  " d.rowId,unitId,iddecoder,d.unit,a.label,cdate,ctime,modelId,d.enabled "
        req +=  " FROM unitIds as d LEFT JOIN alias as a ON d.unit = a.unit "
        req +=  " WHERE d.typeunit = '" + param + "';"
        if rowids: req += " AND d.rowId IN (" + rowids + ")"
    elif param == 'allparam':
        req +=  " d.rowId,modaddrId,parameter,unit,updatedmaxt,d.enabled "
        req +=  "FROM allparamIds as d, allunitids as a WHERE modaddrid = unitid "
        req +=  " AND a.typeunit = 'sensor' "
        if rowids: req += " AND d.rowId IN (" + rowids + ")"
    elif param == 'parameter':
        req +=  " d.rowId,modaddrId,parameter,unit,updatedmaxt,d.enabled "
        req +=  "FROM parameterIds as d, unitids as a WHERE modaddrid = unitid "
        req +=  " AND a.typeunit = 'sensor' "
        if rowids: req += " AND d.rowId IN (" + rowids + ")"
    rows = ralldb3(req,())
    if (not rows) and (not rowids) : return 'No rows of ' + param
    return '\n'.join(dumprows(rows))


def enable(data):
    return updunit('True',data)


def disable(data):
    return updunit('False',data)


def updparam(truefalse,item):
    """
    Update field enabled to 'True' or 'False' 
    for table parameterIds
    truefalse = 'True' or 'False'
    item = [type,list of rowids]
    type = ['sensor']
    """
    rowids = ','.join(item[1:])
    if item[0] == 'sensor':
        req0 = "SELECT unitId FROM allunitids WHERE rowId IN (" + rowids + ")"
        rows = ralldb3(req0,())
        uids = ["'" + r[0] + "'" for r in rows]
        cond =  " WHERE modaddrId IN (" + ",".join(uids) + ")"
        req = "UPDATE allparamids SET enabled = ? "
        req +=  cond
        if truefalse == 'False':
            req1 = "DELETE FROM parameterids " + cond
        else:
            req1 = "INSERT INTO parameterids SELECT * FROM allparamids " + cond
        try:
            majdb3(req,(truefalse,))
            majdb3(req1,())
        except:
            pass


def updunit(truefalse,data):
    """
    Update field enabled to 'True' or 'False' 
    for table parameterIds, unitIds,  allunitids
    truefalse = 'True' or 'False'
    item = [type,list of rowids]
    type = ['alias','detector','device','parameter','remote','sensor']
    """
    if isinstance(data,list):
        item = data
    else:
        item = data.split()
    type = ['alias','detector','device','parameter','remote','sensor']
    if item[0] not in type:
        return 'First parameter must be in the list %s' % repr(type)
    if len(item) < 2:
        return 'Syntax error, minimum of one rowId or parameter keyword needed after first argument'
    #Check if item is integer for rowId
    for i in range(1,len(item)):
        if not item[i].isdigit(): return 'Syntax error, this parameter %s must be an integer' % repr(item[i])
    req = "UPDATE "
    req1 = None
    if item[0] == 'alias':
        req += " alias "
    elif item[0] in ('detector','device','remote','sensor'):
        req += " allunitids "
        if truefalse == 'False':
            req1 = "DELETE FROM unitids "
        else:
            req1 = "INSERT INTO unitids SELECT * FROM allunitids "
    elif item[0] == 'parameter':
        req += " allparamids "
        if truefalse == 'False':
            req1 = "DELETE FROM parameterids "
        else:
            req1 = "INSERT INTO parameterids SELECT * FROM allparamids "
    req += " SET enabled = ? "
    rowids = ','.join(item[1:])
    req += " WHERE rowId IN (" + rowids + ")"
    req1 += " WHERE rowId IN (" + rowids + ")"
    try:
        nb = majdb3(req,(truefalse,))
        if req1: majdb3(req1,())
    except Exception as e:
        return 'Error update :%s' % repr(e)
    #For sensor
    updparam(truefalse,item)
    if nb > 0:
        return 'Update successful for %s rows ' % (nb)
    else:
        return 'No update made with %s' % repr (data)
